
#include "../include/defines.h"

#include "device.h"
#include "hook.h"


#include <linux/module.h>

//dev_t type
#include <linux/types.h>

//MAJOR() and MINOR()
#include <linux/kdev_t.h>

//Device related structure and functions
#include <linux/fs.h>
#include <linux/cdev.h>

//access_ok, put_user
#include <asm/uaccess.h>

//for_each_process
#include <linux/sched.h>

//Device major and minor numbers
static dev_t g_device_num;

//Count the number of hooked interrupts
extern volatile unsigned int nb_interrupts;

//The file_operation structure, to link the device to the appropriate handlers
static struct file_operations g_fops = {
   .owner   = THIS_MODULE,
   .ioctl   = my_ioctl,
};


//Char device structure
static struct cdev g_device;

/**
 * Create the device
 */
int create_device (void)
{
   //Allocate the device major and minor
   if(alloc_chrdev_region(&g_device_num, 0, 1, DEVICE_NAME))
   {
      printk(KERN_INFO "ERROR: alloc_chrdev_region FAILED\n");
      return -1;
   }

   //Initialise the device
   cdev_init(&g_device, &g_fops);

   //Fill in some fields (optional)
   g_device.owner = THIS_MODULE;
   g_device.ops = &g_fops;

   //Register the device into the kernel
   if(cdev_add(&g_device, g_device_num, 1))
   {
      printk(KERN_INFO "ERROR: cdev_add FAILED\n");
      return -1;
   }

   printk(KERN_INFO "Device registrated successfully - name = %s, major = %d, minor = %d\n", DEVICE_NAME, MAJOR(g_device_num), MINOR(g_device_num));

   return 0;
}

/**
 * Delete the device
 */
void delete_device (void)
{
   //Unregister the device
   cdev_del(&g_device);

   //Unregister the device number
   unregister_chrdev_region(g_device_num, 1);
}


/**
 * IOCTL handler
 */
int my_ioctl (struct inode * _inode, struct file * _file, unsigned int _ioctl_num, unsigned long _ioctl_param)
{
   struct task_struct *task;
   
   switch(_ioctl_num)
   {
      //Set the PID
      case (IOCTL_SET_PID):

         pid_to_hook = (unsigned int) _ioctl_param;
         printk(KERN_INFO "rdtsc_exploit: pid_to_hook = %d.\n", pid_to_hook);
         printk(KERN_INFO "rdtsc_exploit: nb_interrupts = %d.\n", nb_interrupts);
         printk(KERN_INFO "rdtsc_exploit: GetCurrentPID() = %d.\n", GetCurrentPID());
         printk(KERN_INFO "rdtsc_exploit: CR4.TSD = %08x.\n", GetCR4() & FLAG_DISABLE_USER_RDTSC);
         
         for_each_process(task) {
            printk("%s[%d], TIF_NOTSC = %d\n", task->comm, task->pid, test_ti_thread_flag(task_thread_info(task), TIF_NOTSC));
            if(task->pid == pid_to_hook){
               test_and_set_ti_thread_flag(task_thread_info(task), TIF_NOTSC);
               printk("===> TIF_NOTSC set for process %d <===\n", task->pid);
            }
         }
         

      break;

      default:
         printk(KERN_INFO "rdtsc_exploit: ERROR: Unsupported ioctl code: %08x.\n", _ioctl_num);
   }
   return 0;
}
