#ifndef _HOOK_H
#define _HOOK_H

#include "../include/defines.h"

//Flag of CR4 that disable RDTSC in userland
#define FLAG_DISABLE_USER_RDTSC 0x4

extern volatile unsigned int nb_interrupts;
extern volatile unsigned int pid_to_hook;

#define IDT_HANDLER(p_desc) ((((P_IDTENTRY_ST)p_desc)->hi_offset << 16) | ((P_IDTENTRY_ST)p_desc)->low_offset)
#define IDT_HANDLER_IV(p_idt, iv) IDT_HANDLER(((P_IDTENTRY_ST)p_idt) + iv)

/**
 * An IDT entry. Cf Intel SDM 3A
 */
typedef struct {
   u_int16 low_offset;
   u_int16 selector;
   u_int8 unused_lo;
   u_int8 segment_type:4;
   u_int8 system_segment_flag:1;
   u_int8 DPL:2;
   u_int8 P:1;
   u_int16 hi_offset;
} __attribute__((packed)) IDTENTRY_ST, *P_IDTENTRY_ST;


/**
 * Interrupt stack structures
 */
typedef struct
{
   u_int32 edi;
   u_int32 esi;
   u_int32 ebp;
   u_int32 esp;
   u_int32 ebx;
   u_int32 edx;
   u_int32 ecx;
   u_int32 eax;
} PUSHA_ST, *P_PUSHA_ST;

typedef struct
{
   u_int32 error_code;  // !! Check Intel Manuals to see if the error code is present or not
   u_int32 eip;
   u_int32 cs;
   u_int32 eflags;
   u_int32 esp;
   u_int32 sp;
} INT_STACK_HARD_ST, P_INT_STACK_HARD_ST;

typedef struct
{
   u_int32              eflags;

   PUSHA_ST             pusha_st;
   INT_STACK_HARD_ST    int_stack_hard_st;

} MY_INT_STACK_ST, *P_MY_INT_STACK_ST;


/**
 * Struct for RDTSC
 */
typedef struct {
   u_int32 eax;
   u_int32 edx;
} RDTSC_ST, *P_RDTSC_ST;


/**
 * Get the IDT address of the current processor
 */
P_IDTENTRY_ST GetIDTHard (void);

/**
 * Get the IDT address of the current processor
 */
P_IDTENTRY_ST GetIDTSoft (void);


/**
 * Get CR4 value
 */
u_int32 GetCR4 (void);

/**
 * Set CR4 value
 */
void SetCR4 (u_int32 _new_cr4);

/**
 * Enable userland calls to RDTSC
 */
void EnableUserRDTSC (void);

/**
 * Disable userland calls to RDTSC
 */
void DisableUserRDTSC (void);

/**
 * Return current PID
 */
unsigned int GetCurrentPID (void);

/**
 * Hook
 */
void Hook (void);

/**
 * Unhook
 */
void UnHook (void);


/**
 * Hook an interrupt handler
 */
void HookOneIDT (P_IDTENTRY_ST _idt_inter, u_int32 _interrupt_number, u_int32* _old_address, u_int32 _new_address);

/**
 * The interrupt handler, coded in ASM
 */
extern void interrupt_handler (void);


/**
 * Function called by the interrupt handler
 */
u_int32  __attribute__((__fastcall__)) my_func_handler (P_MY_INT_STACK_ST stack);

#endif
